@@ -1076,6 +1076,16 @@ def _refine_corr(
             GNNMS[key] = gnnm3
     return GNNMS
 
+def _prepend_blast_prefix(data, pre):
+    x = [str(x).split("_")[0] for x in data]
+    vn = []
+    for i,g in enumerate(data):
+        if x[i] != pre:
+            vn.append(pre+"_"+g)
+        else:
+            vn.append(g)
+    return np.array(vn).astype('str').astype('object')
+
 def _calculate_blast_graph(ids, f_maps="maps/", eval_thr=1e-6, reciprocate=False):
     gns = []
     Xs=[]
@@ -1111,15 +1121,11 @@ def _calculate_blast_graph(ids, f_maps="maps/", eval_thr=1e-6, reciprocate=False
                 B = B[B.index.astype("str") != "nan"]
                 B = B[B.iloc[:, 0].astype("str") != "nan"]
 
-                A.index = id1 + "_" + A.index.astype("str").astype("object")
-                B.iloc[:, 0] = (
-                    id1 + "_" + B.iloc[:, 0].values.flatten().astype("str").astype("object")
-                )
+                A.index = _prepend_blast_prefix(A.index,id1)
+                B.iloc[:, 0] = _prepend_blast_prefix(B.iloc[:, 0].values.flatten(),id1)
 
-                B.index = id2 + "_" + B.index.astype("str").astype("object")
-                A.iloc[:, 0] = (
-                    id2 + "_" + A.iloc[:, 0].values.flatten().astype("str").astype("object")
-                )
+                B.index = _prepend_blast_prefix(B.index,id2)
+                A.iloc[:, 0] = _prepend_blast_prefix(A.iloc[:, 0].values.flatten(),id2)
 
                 i1 = np.where(A.columns == "10")[0][0]
                 i3 = np.where(A.columns == "11")[0][0]
@@ -1282,26 +1288,26 @@ def prepare_SAMap_loadings(sam, npcs=300):
 
 
 def _concatenate_sam(sams, nnm):
-
     acns = []
-    obsks = []
     exps = []
     agns = []
+    sps = []
     for i,sid in enumerate(sams.keys()):
         acns.append(q(sams[sid].adata.obs_names))
-        obsks.append(np.array(sams[sid].adata.obs_keys()))
+        sps.append([sid]*acns[-1].size)
         exps.append(sams[sid].adata.X)
         agns.append(q(sams[sid].adata.var_names))
-    
-    obsk = np.unique(np.concatenate(obsks))
+
 
     acn = np.concatenate(acns)
     agn = np.concatenate(agns)
+    sps = np.concatenate(sps)
+
     xx = sp.sparse.block_diag(exps,format='csr')
-    
-    
+
+
     sam = SAM(counts=(xx, agn, acn))
-    
+
     sam.adata.uns["neighbors"] = {}
     nnm = nnm.tocsr()
     nnm.eliminate_zeros()
@@ -1312,20 +1318,12 @@ def _concatenate_sam(sams, nnm):
         "use_rep": "X",
         "metric": "euclidean",
     }
-
-    for k in obsk:
-        ann = []
-        for i in sams.keys():
-            if k in sams[i].adata.obs.keys():
-                ann.append(
-                    sams[i].adata.var_names[0].split("_")[0]
-                    + "_"
-                    + sams[i].get_labels(k).astype('str').astype("object")
-                )
-            else:
-                ann.append(q([""] * sams[i].adata.shape[0]))
-
-        sam.adata.obs[k] = pd.Categorical(np.concatenate(ann))
+    for i in sams.keys():
+        for k in sams[i].adata.obs.keys():
+            if sams[i].adata.obs[k].dtype.name == "category":
+                z = np.array(['unassigned']*sam.adata.shape[0],dtype='object')
+                z[sps==i] = q(sams[i].adata.obs[k])
+                sam.adata.obs[i+'_'+k] = pd.Categorical(z)
 
     a = []
     for i,sid in enumerate(sams.keys()):
